
# q-expansion of newform 1936.4.a.t, downloaded from the LMFDB on 30 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1936
weight = 4
poly_data = [-1, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[-2, 4], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [1, 1], [-3, 4], [-28, -1], [0, 0], [22, 11], [4, 15], [-94, -9], [33, -29], [28, -46], [-103, 7], [71, -52], [-126, -13], [-186, 28], [100, 18], [-530, -14], [391, -47], [120, 70], [-141, 65], [-377, 15], [-34, -161], [26, 88], [2, 166], [139, -186], [153, -104], [800, -213], [516, -18], [-184, 135], [-154, 188], [1493, -38], [-626, -323], [-1400, 57], [-99, -206], [-2080, 95], [-1184, 41], [-78, -594], [-1355, -536], [-620, -336], [-1810, 519], [2606, 176], [319, 215], [3063, 318], [575, -319], [-892, -436], [2122, 368], [1184, 368], [724, -520], [4305, 89], [2176, 467], [-1049, -822], [-2228, -306], [4386, -322], [-3916, -374], [-7237, -79], [-2902, -382], [2772, -769], [274, -1008], [726, 187], [2622, -858], [-2402, -334], [-5704, 90], [-42, 201], [-526, -437], [-2112, 934], [-4165, 526], [-1511, 1976], [5429, 335], [2344, -969], [-8390, 549], [2052, 379], [431, 150], [6486, -457], [785, 71], [-10908, -17], [4245, 353], [8325, -1347], [6201, 814], [-750, 866], [4286, 698], [5310, 568], [5008, -1222], [-2870, -1654], [5912, -91], [7389, 1716], [1116, 1982], [-6337, 669], [6783, 850], [6080, -1418], [2736, 2938], [-4337, -597], [879, 161], [-8974, 69], [-1187, -4255], [-9872, 812], [7388, 2788], [-11126, -160], [10999, 614], [-8235, 1536], [4158, 33], [16510, -1195], [5324, -3322], [15936, -739], [1088, 3062], [-4700, -98], [6228, 2588], [7077, -1394], [4376, 4034], [-8604, -3864], [7360, 2724], [-8580, -2951], [-3386, -1784], [-5128, 2428], [-3742, -1928], [-6951, -43], [-13343, 1695], [423, -1324], [8019, -2797], [12239, 2537], [4847, -3514], [-1074, 1688], [-9123, 1574], [-382, -3032], [1424, -447], [11580, 3210], [-5169, -4065], [-12208, 667], [11327, 730], [7893, 2373], [-3095, -5535], [-1752, -1418], [-6558, -4134], [7108, 5052], [19003, -1009], [30114, 944], [-3718, 5005], [22294, -3013], [2554, -3660], [292, 5278], [-11651, 5714], [-10310, 6403], [-20130, -2336], [10018, -920], [9261, -559], [314, 2253], [-15419, 1232], [-10903, 1949], [1962, -1576], [28720, 3876], [-15625, 3423], [-11752, -2272], [30026, -2151], [-36111, 370], [-18328, -146], [38534, 2320], [-31596, -248], [12152, 954], [2350, 5100], [-5082, -9130], [-8782, -3933], [28050, 790], [-30901, -5397], [-8176, -1921], [-11524, -9630], [-2771, 1223], [465, -11230], [-18339, -3007], [-28344, 4520], [19552, 5873], [-19074, 4726], [17215, 4418], [10830, 5858], [-31762, 2000], [9710, -3343], [26018, 2023], [-12217, 9551], [-855, 3100], [-32556, 1801], [-1161, -1876], [21060, 3496], [22132, 9944], [-1880, -6560], [-8186, -9018], [2049, -4600], [-31186, -1466], [-7593, 5627], [30934, -910], [23970, -2105], [8628, 11938], [21252, 685], [-31070, -4450], [56085, -3066], [-4232, 7879], [6983, -11391], [-17154, 8028], [53262, 2255], [-40777, -6394], [17264, -1929], [-23177, 2250], [-43146, -3094], [-17568, -9495], [1446, -10041], [61418, 36], [24578, 4252], [26228, 5858], [-40379, 9999], [-35171, -7160], [36283, 6651], [-25200, -3678], [-13268, -9078], [6980, 11898], [-40496, 9216], [-59033, -4696], [8265, -4221], [19048, -6518], [-32520, 11411], [67295, -6012], [78466, -859], [11888, -7182], [19220, -7734], [18919, 3042], [5514, -6491], [-48330, 8399], [3305, 8828], [-63517, 1685], [-67674, -3677], [-6830, 2822], [-50458, -8168], [11308, -9614], [-71598, -1775], [18918, -15293], [-13003, 1848], [18762, -2987], [19610, 5032], [-64612, 3666], [-103553, 291], [-9278, 16866], [77061, -2762], [-57450, 10031], [-1975, -9507], [-9941, -9671], [-48667, 12923], [2904, -2508], [32136, 16602], [-31419, 8084], [7576, -7600], [-20472, 554], [36416, -10842], [-8876, -2512], [-734, -1334], [-77826, 6880], [3664, -136], [-15788, 14677], [-2261, 4087], [19434, 20230], [60572, 5385], [-29560, -12594], [-60634, 6740], [41892, 15363], [31277, 12588], [51376, 4593], [-67816, -6046], [29034, -1285], [6828, -22707], [-31952, -10593], [53698, 6946], [52860, -7274], [-59985, 6004], [86973, 9484], [9758, -16908], [45226, 4554], [1433, -5870], [-54083, -3105], [-28850, 14618], [-62250, -13119], [-3538, 7189], [3677, -1237], [59369, -18683], [-42350, -3578], [-56110, 14637], [8782, -4141], [12114, 4267], [-75317, -239], [-72850, -2069], [-59210, 17418], [6128, 27650], [10287, 17431], [-73398, -3936], [-32740, -3771], [-11321, 19711], [1320, -17431], [-9127, -32680], [-106944, -12466], [82604, 4556], [1660, -7177], [-8836, 29507], [-97154, 13813], [43075, -2085], [-88173, 3892], [3876, 17627], [100726, -9772], [92372, -11905], [-39470, -12741], [-72680, 2542], [22852, 17096], [81317, 4917], [35595, -10590], [-5811, -6561], [-40879, 460], [-73008, 534], [-14286, 1077], [-23868, -14814], [51142, 3684], [-55230, 23256], [-80581, -14827], [-45392, 26336], [71972, 13995], [-42895, 28043], [-29282, 7028], [-55954, -18426], [12586, 19121], [-20042, -19305], [11953, -19712], [69036, -14605], [-45517, 4889], [28094, 29014], [-71789, -392], [106289, 11771], [14465, 17253], [-47158, -25461], [-53452, -2628], [43018, -14980], [-29798, 2614], [-15164, 4873], [99164, 2622], [15660, -25966], [116739, -11241], [-115719, -922], [-89916, 140], [1763, -29834], [-16776, -17927], [56625, -19208], [-3881, -3420], [2062, -141], [-119003, -15365], [56567, 27396], [13722, 24546], [-76637, 24686], [27441, -27637], [80522, 18395], [-19601, -21118], [63394, -7715], [-145845, 10152], [-111098, -25808], [93796, -1360], [147160, -7119], [-187290, 5690], [-10245, 493], [11326, -13002], [13690, -39730], [52487, -15055], [52610, 33692], [75148, -28275], [162793, -1515], [26816, 1909], [16537, -6721], [-184907, 3132], [105546, -2589], [66522, -40652], [10406, -40535], [-409, 37335], [92388, -1114], [-128828, -25883], [-51150, -26807], [109766, 14454], [68621, -24770], [-128616, -17122], [10052, 14563], [125922, -22104], [62234, -13630], [-34832, 19319], [-38350, 37966], [-77445, -23950], [-129696, 28093], [-8364, -31501], [-105208, 18936], [12745, 39851], [38740, -41990], [103937, 21270], [116978, -9465], [-122703, 22019], [-45627, 25128], [-68377, -36778], [-151953, 18455], [-73831, 41957], [-36547, -26135], [110818, -6165], [17788, 50029], [-90870, -12566], [54072, -24646], [-14198, 8540], [-7032, -19129], [-81079, 38684], [16448, -15544], [91970, -10182], [-155328, -1662], [17880, -37985], [103606, 7668], [-53986, -27497], [162572, -1992], [-47447, -5159], [-8742, -36090], [-59676, -46791], [62780, 40028], [-54728, -13985], [40368, -46647], [-13047, 30698], [-84644, 8070], [28037, -12403], [82046, 35300], [99570, 1780], [-176249, -20462], [-7948, 54474], [-108300, -2950], [75212, -15217], [-39430, -9992], [-173501, -30982], [-55671, -24179], [-110120, 26441], [269005, -9035], [96256, 16520]]
